CSECT   REALV1
 
      SUBROUTINE REALV1
     *  (N,NM,M,IVEC,A,VECR,LAMDA,IWORK,WORK,INDIC,EPS,EX)
 
C THIS SUBROUTINE FINDS THE REAL EIGENVECTOR OF THE REAL
C UPPER-HESSENBERG MATRIX IN THE ARRAY A, CORRESPONDING TO
C THE REAL EIGENVALUE STORED IN EVR(IVEC). THE INVERSE
C ITERATION METHOD IS USED.
C NOTE THE MATRIX IN A IS DESTROYED BY THE SUBROUTINE.
C N IS THE ORDER OF THE UPPER-HESSENBERG MATRIX.
C NM DEFINES THE FIRST DIMENSION OF THE TWO DIMENSIONAL
C ARRAYS A AND VECR. NM MUST BE EQUAL TO OR GREATER THAN N.
C M IS THE ORDER OF THE SUBMATRIX OBTAINED BY A SUITABLE
C DECOMPOSITION OF THE UPPER-HESSENBERG MATRIX IF SOME
C SUBDIAGONAL ELEMENTS ARE EQUAL TO ZERO. THE VALUE OF M IS
C CHOSEN SO THAT THE LAST N-M COMPONENTS OF THE EIGENVECTOR
C ARE ZERO.
C IVEC GIVES THE POSITION OF THE EIGENVALUE IN THE ARRAY EVR
C FOR WHICH THE CORRESPONDING EIGENVECTOR IS COMPUTED.
C THE ARRAY EVI WOULD CONTAIN THE IMAGINARY PARTS OF THE N
C EIGENVALUES IF THEY EXISTED.
C
C THE M COMPONENTS OF THE COMPUTED REAL EIGENVECTOR WILL BE
C FOUND IN THE FIRST M PLACES OF THE COLUMN IVEC OF THE TWO
C DIMENSIONAL ARRAY VECR.
C
C IWORK AND WORK ARE THE WORKING STORES USED DURING THE
C GAUSSIAN ELIMINATION AND BACKSUBSTITUTION PROCESS.
C THE ARRAY INDIC INDICATES THE SUCCESS OF THE ROUTINE AS
C FOLLOWS
C     VALUE OF INDIC(I)     EIGENVECTOR I
C            1                NOT FOUND
C            2                  FOUND
C EPS IS A SMALL POSITIVE NUMBER THAT NUMERICALLY REPRESENTS
C ZERO IN THE PROGRAM. EPS = (EUCLIDIAN NORM OF A)*EX, WHERE
C EX = 2**(-T). T IS THE NUMBER OF BINARY DIGITS IN THE
C MANTISSA OF A FLOATING POINT NUMBER.
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER (I-N)
 
      DOUBLE PRECISION LAMDA (NM)
      DIMENSION A(NM,1),VECR(NM,1),IWORK(NM),WORK(NM),INDIC(NM)
 
      VECR(1,IVEC) = 1.0D0
      IF(M.EQ.1) GO TO 24
 
C SMALL PERTURBATION OF EQUAL EIGENVALUES TO OBTAIN A FULL
C SET OF EIGENVECTORS.
 
      EVALUE=LAMDA(2*IVEC-1)
      IF(IVEC.EQ.M) GO TO 2
      K = IVEC+1
      R = 0.0D0
      DO 1 I=K,M
      IF (EVALUE .NE. LAMDA(2*I-1)) GO TO 1
      IF(LAMDA(2*I) .NE. 0.D0) GO TO 1
      R = R+3.0D0
    1 CONTINUE
      EVALUE = EVALUE+R*EX
    2 DO 3 K=1,M
    3 A(K,K) = A(K,K)-EVALUE
 
C GAUSSIAN ELIMINATION OF THE UPPER-HESSENBERG MATRIX A. ALL
C ROW INTERCHANGES ARE INDICATED IN THE ARRAY IWORK. ALL THE
C MULTIPLIERS ARE STORED AS THE SUBDIAGONAL ELEMENTS OF A.
 
      K = M-1
      DO 8 I=1,K
      L = I+1
      IWORK(I) = 0
      IF(A(I+1,I).NE.0.0D0) GO TO 4
      IF(A(I,I).NE.0.0D0) GO TO 8
      A(I,I) = EPS
      GO TO 8
    4 IF(DABS(A(I,I)).GE.DABS(A(I+1,I))) GO TO 6
      IWORK(I) = 1
      DO 5 J=I,M
      R = A(I,J)
      A(I,J) = A(I+1,J)
    5 A(I+1,J) = R
    6 R = -A(I+1,I)/A(I,I)
      A(I+1,I) = R
      DO 7 J=L,M
    7 A(I+1,J) = A(I+1,J)+R*A(I,J)
    8 CONTINUE
      IF(A(M,M).NE.0.0D0) GO TO 9
      A(M,M) = EPS
 
C THE VECTOR (1,1,...,1) IS STORED IN THE PLACE OF THE RIGHT
C HAND SIDE COLUMN VECTOR.
 
    9 DO 11 I=1,N
      IF(I.GT.M) GO TO 10
      WORK(I) = 1.0D0
      GO TO 11
   10 WORK(I) = 0.0D0
   11 CONTINUE
 
C THE INVERSE ITERATION IS PERFORMED ON THE MATRIX UNTIL THE
C INFINITE NORM OF THE RIGHT-HAND SIDE VECTOR IS GREATER
C THAN THE BOUND DEFINED AS 0.01D0/(N*EX)
 
      BOUND = 0.01D0/(EX*dfloat(N))
      NS = 0
      ITER = 1
 
C THE BACKSUBSTITUTION.
 
   12 R = 0.0D0
      DO 15 I=1,M
      J = M-I+1
      S = WORK(J)
      IF(J.EQ.M) GO TO 14
      L = J+1
      DO 13 K=L,M
      SR = WORK(K)
   13 S = S-SR*A(J,K)
   14 WORK(J) = S/A(J,J)
      T = DABS(WORK(J))
      IF(R.GE.T) GO TO 15
      R = T
   15 CONTINUE
 
C THE COMPUTATION OF THE RIGHT-HAND SIDE VECTOR FOR THE NEW
C ITERATION STEP.
 
      DO 16 I=1,M
   16 WORK(I) = WORK(I)/R
 
C THE COMPUTATION OF THE RESIDUALS AND COMPARISON OF THE
C RESIDUALS OF THE TWO SUCCESSIVE STEPS OF THE INVERSE
C ITERATION. IF THE INFINITE NORM OF THE RESIDUAL VECTOR IS
C GREATER THAN THE INFINITE NORM OF THE PREVIOUS RESIDUAL
C VECTOR THE COMPUTED EIGENVECTOR OF THE PREVIOUS STEP IS
C TAKEN AS THE FINAL EIGENVECTOR.
 
      R1 = 0.0D0
      DO 18 I=1,M
      T = 0.0D0
      DO 17 J=I,M
   17 T = T+A(I,J)*WORK(J)
      T = DABS(T)
      IF(R1.GE.T) GO TO 18
      R1 = T
   18 CONTINUE
      IF(ITER.EQ.1) GO TO 19
      IF(PREVIS.LE.R1) GO TO 24
   19 DO 20 I=1,M
   20 VECR(I,IVEC) = WORK(I)
      PREVIS = R1
      IF(NS.EQ.1) GO TO 24
      IF(ITER.GT.6) GO TO 25
      ITER = ITER+1
      IF(R.LT.BOUND) GO TO 21
      NS = 1
 
C GAUSSIAN ELIMINATION OF THE RIGHT-HAND SIDE VECTOR.
 
   21 K = M-1
      DO 23 I=1,K
      R = WORK(I+1)
      IF(IWORK(I).EQ.0) GO TO 22
      WORK(I+1) = WORK(I)+WORK(I+1)*A(I+1,I)
      WORK(I) = R
      GO TO 23
   22 WORK(I+1) = WORK(I+1)+WORK(I)*A(I+1,I)
   23 CONTINUE
      GO TO 12
 
   24 INDIC(IVEC) = 2
   25 IF(M.EQ.N) GO TO 27
      J = M+1
      DO 26 I=J,N
   26 VECR(I,IVEC) = 0.0D0
   27 RETURN
 
      END
